@@ -12,9 +12,6 @@ pub struct Args {
         required = false)]
     pub multiparts: Vec<PathBuf>,
 
-    #[arg(short, long, help = "Path to the startup script.", required = false)]
-    pub script: Option<PathBuf>,
-
     #[arg(
         short,
         long,
@@ -104,17 +101,6 @@ pub struct Args {
     )]
     pub truncate_ragged_lines: bool,
 
-    #[arg(long, help = "Tabiew theme", required = false, value_enum, default_value_t = AppTheme::Monokai)]
-    pub theme: AppTheme,
-
-    #[arg(
-        long,
-        help = "Generate a sample theme file in $HOME/.config/tabiew",
-        required = false,
-        default_value_t = false
-    )]
-    pub generate_theme: bool,
-
     #[arg(
         long,
         help = "Specifies the types to infer for text-based files.",
@@ -135,6 +121,8 @@ pub struct Args {
 #[derive(Debug, Clone, ValueEnum)]
 pub enum Format {
     Dsv,
+    Csv,
+    Tsv,
     Parquet,
     Jsonl,
     Json,
@@ -142,6 +130,7 @@ pub enum Format {
     Fwf,
     Sqlite,
     Excel,
+    Logfmt,
 }
 
 #[derive(Debug, Clone)]
@@ -217,18 +206,6 @@ pub enum InferSchema {
     Safe,
 }
 
-#[derive(Debug, Clone, ValueEnum)]
-pub enum AppTheme {
-    Monokai,
-    Argonaut,
-    Nord,
-    Catppuccin,
-    TokyoNight,
-    Chakra,
-    Terminal,
-    Config,
-}
-
 impl InferSchema {
     pub fn to_csv_infer_schema_length(&self) -> Option<usize> {
         match self {
