@@ -1,9 +1,13 @@
+pub mod config;
 pub mod globals;
-pub mod history;
 pub mod iter_ext;
+pub mod jagged_vec;
+pub mod non_empty_stack;
+pub mod osc52;
+pub mod paths;
 pub mod polars_ext;
 pub mod search;
+pub mod snake_case_name_gen;
 pub mod sql;
 pub mod type_ext;
 pub mod type_inferer;
-pub mod vec_map;
