@@ -1,13 +1,16 @@
 mod excel;
 mod fwf;
+mod logfmt;
 mod sqlite;
 
 use anyhow::{Ok, anyhow};
-use excel::ExcelToDataFarmes;
+pub use excel::ExcelToDataFarmes;
 pub use fwf::FwfToDataFrame;
+pub use logfmt::LogfmtToDataFrame;
 pub use sqlite::SqliteToDataFrames;
 
 use std::{
+    borrow::Cow,
     ffi::OsStr,
     fs::File,
     path::{Path, PathBuf},
@@ -47,19 +50,10 @@ impl Source {
         }
     }
 
-    pub fn display_path(&self) -> String {
+    pub fn display_path(&self) -> Cow<'_, str> {
         match self {
-            Source::File(path_buf) => path_buf.to_string_lossy().into_owned(),
-            Source::Stdin => "Stdin".to_owned(),
-        }
-    }
-}
-
-impl From<String> for Source {
-    fn from(value: String) -> Self {
-        match value.as_str() {
-            "$stdin" => Source::File(value.into()),
-            _ => Source::Stdin,
+            Source::File(path_buf) => path_buf.file_name().unwrap_or_default().to_string_lossy(),
+            Source::Stdin => Cow::Borrowed("Stdin"),
         }
     }
 }
@@ -75,14 +69,20 @@ pub trait BuildReader {
 impl BuildReader for Args {
     fn build_reader(&self, path: impl AsRef<Path>) -> AppResult<Box<dyn ReadToDataFrames>> {
         match self.format {
-            Some(Format::Dsv) => Ok(Box::new(CsvToDataFrame::from_args(self))),
+            Some(Format::Dsv) | Some(Format::Csv) => Ok(Box::new(CsvToDataFrame::from_args(self))),
+            Some(Format::Tsv) => {
+                let mut reader = CsvToDataFrame::from_args(self);
+                reader.separator_char = '\t';
+                Ok(Box::new(reader))
+            }
             Some(Format::Parquet) => Ok(Box::new(ParquetToDataFrame)),
             Some(Format::Json) => Ok(Box::new(JsonToDataFrame::from_args(self))),
             Some(Format::Jsonl) => Ok(Box::new(JsonLineToDataFrame::from_args(self))),
             Some(Format::Arrow) => Ok(Box::new(ArrowIpcToDataFrame)),
-            Some(Format::Fwf) => Ok(Box::new(FwfToDataFrame::from_args(self)?)),
+            Some(Format::Fwf) => Ok(Box::new(FwfToDataFrame::from_args(self))),
             Some(Format::Sqlite) => Ok(Box::new(SqliteToDataFrames::from_args(self))),
             Some(Format::Excel) => Ok(Box::new(ExcelToDataFarmes::from_args(self))),
+            Some(Format::Logfmt) => Ok(Box::new(LogfmtToDataFrame::from_args(self))),
             None => match path.as_ref().extension().and_then(|ext| ext.to_str()) {
                 Some("tsv") => {
                     let mut reader = CsvToDataFrame::from_args(self);
@@ -93,7 +93,7 @@ impl BuildReader for Args {
                 Some("json") => Ok(Box::new(JsonToDataFrame::from_args(self))),
                 Some("jsonl") => Ok(Box::new(JsonLineToDataFrame::from_args(self))),
                 Some("arrow") => Ok(Box::new(ArrowIpcToDataFrame)),
-                Some("fwf") => Ok(Box::new(FwfToDataFrame::from_args(self)?)),
+                Some("fwf") => Ok(Box::new(FwfToDataFrame::from_args(self))),
                 Some("db") | Some("sqlite") => Ok(Box::new(SqliteToDataFrames::from_args(self))),
                 Some("xls") | Some("xlsx") | Some("xlsm") | Some("xlsb") => {
                     Ok(Box::new(ExcelToDataFarmes::from_args(self)))
